require 'thor'
require 'yaml'
require 'openssl'
require 'fog'

class Titantic < Thor

  def initialize(args = [], options = {}, configs = {})
    super
   
    if File.exists?('.titantic')
      config = YAML.load_file('.titantic')
    elsif File.exists?(File.join(Dir.home, '.titantic'))
      config = YAML.load_file(File.join(Dir.home, '.titantic')) 
    else
      if(ENV['TITANTIC_CONFIG'])
        File.exists?(ENV['TITANTIC_CONFIG'])
        config = YAML.load_file(ENV['TITANTIC_CONFIG'])  
      end
    end

    unless config
      puts "Could not load configuration."
      puts "Titantic configuration is YAML format and must exist in 1 of 3 places"
      puts "1. in the current directory as a file named '.titantic' "
      puts "2. in the current user's home directory as a file named '.titantic' "
      puts "3. as a file path specified by the environment variable 'TITANTIC_CONFIG'"
    end

    @glacier = Fog::AWS::Glacier.new config
  end
  
  desc 'upload [VAULT] [BODY] [DESCRIPTION]', 'upload the given archive into the given vault'
  def upload_archive(vault_name, body, options = {})
    vault = @glacier.vaults.get vault_name
    unless vault
      print Hash[ status: 'error', error: "Vault #{vault_name} does not exist" ].to_json 
      return 
    end

    unless File.exists?(body)
      print Hash[ status: 'error', error: "Archive #{body} does not exist" ].to_json 
      return 
    end

    body = File.read(body)
    response = @glacier.create_archive(vault_name, body, options)

    if response.status == 201
      print Hash[ status: 'success' ].to_json 
      return 
    end

    print [ status: 'error', error: response.headers ].to_json
  end

  desc 'create_vault [VAULT]', 'create a new vault'
  def create_vault(vault_name)
    begin
      vault = @glacier.vaults.create :id => vault_name
    rescue Exception => e
        print [ status: 'error', error: e.to_s]
    end
    print vault.to_json
  end

  desc 'delete_vault [VAULT]', 'delete a vault with the given name'
  def delete_vault(vault_name)
    vault = @glacier.vaults.get vault_name
    unless vault 
      print Hash[ status: 'error', error: "Vault #{vault_name} does not exist" ].to_json 
      return
    end
    print @glacier.delete_vault(vault_name).data.to_json
  end

  desc 'list_vaults', 'retrieve a listing of vault names'
  def list_vaults
    response = @glacier.list_vaults
    print response.data[:body]["VaultList"].collect{ |v| v["VaultName"] }.to_json
  end
end

Titantic.start(ARGV)
